from datetime import datetime

from .._utils import date, http
from ..dataclasses import Channel, Redemption, Reward, User

ENDPOINT_CUSTOM_REWARDS = "https://api.twitch.tv/helix/channel_points/custom_rewards"


def create_custom_reward(
    token: str,
    client_id: str,
    broadcaster_id: str,
    title: str,
    cost: int,
    prompt: str | None = None,
    is_enabled: bool = True,
    background_color: str | None = None,
    is_user_input_required: bool = False,
    is_max_per_stream_enabled: bool = False,
    max_per_stream: int | None = None,
    is_max_per_user_per_stream_enabled: bool = False,
    max_per_user_per_stream: int | None = None,
    is_global_cooldown_enabled: bool = False,
    global_cooldown_seconds: int | None = None,
    should_redemptions_skip_request_queue: bool = False,
) -> Reward:
    url = ENDPOINT_CUSTOM_REWARDS
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {
        "broadcaster_id": broadcaster_id,
        "title": title,
        "cost": cost,
        "is_enabled": is_enabled,
        "is_user_input_required": is_user_input_required,
        "is_max_per_stream_enabled": is_max_per_stream_enabled,
        "is_max_per_user_per_stream_enabled": is_max_per_user_per_stream_enabled,
        "is_global_cooldown_enabled": is_global_cooldown_enabled,
        "should_redemptions_skip_request_queue": should_redemptions_skip_request_queue,
    }

    if prompt is not None:
        params["prompt"] = prompt

    if background_color is not None:
        params["background_color"] = background_color

    if max_per_stream is not None:
        params["max_per_stream"] = max_per_stream

    if max_per_user_per_stream is not None:
        params["max_per_user_per_stream"] = max_per_user_per_stream

    if global_cooldown_seconds is not None:
        params["global_cooldown_seconds"] = global_cooldown_seconds

    reward = http.send_get(url, headers, params)[0]

    return Reward(
        Channel(
            User(
                reward["broadcaster_id"],
                reward["broadcaster_login"],
                reward["broadcaster_name"],
            )
        ),
        reward["id"],
        reward["title"],
        reward["prompt"],
        reward["cost"],
        reward["image"],
        reward["default_image"],
        reward["background_color"],
        reward["is_enabled"],
        reward["is_user_input_required"],
        (
            reward["max_per_stream_setting"]["is_enabled"],
            reward["max_per_stream_setting"]["max_per_stream"],
        ),
        (
            reward["max_per_user_per_stream_setting"]["is_enabled"],
            reward["max_per_user_per_stream_setting"]["max_per_user_per_stream"],
        ),
        (
            reward["global_cooldown_setting"]["is_enabled"],
            reward["global_cooldown_setting"]["global_cooldown_seconds"],
        ),
        reward["is_paused"],
        reward["is_in_stock"],
        reward["should_redemptions_skip_request_queue"],
        reward["redemptions_redeemed_current_stream"],
        datetime.strptime(reward["cooldown_expires_at"], date.RFC3339_FORMAT),
    )


def delete_custom_reward(
    token: str, client_id: str, broadcaster_id: str, reward_id: str
) -> None:
    url = ENDPOINT_CUSTOM_REWARDS
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    data = {"broadcaster_id": broadcaster_id, "id": reward_id}

    http.send_delete(url, headers, data)


def get_custom_reward(
    token: str,
    client_id: str,
    broadcaster_id: str,
    reward_ids: list[str] | None = None,
    only_manageable_rewards: bool = False,
) -> list[Reward]:
    url = ENDPOINT_CUSTOM_REWARDS
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {
        "broadcaster_id": broadcaster_id,
        "only_manageable_rewards": only_manageable_rewards,
    }

    if reward_ids is not None and len(reward_ids) > 0:
        params["id"] = reward_ids

    rewards = http.send_get(url, headers, params)

    return [
        Reward(
            Channel(
                User(
                    reward["broadcaster_id"],
                    reward["broadcaster_login"],
                    reward["broadcaster_name"],
                )
            ),
            reward["id"],
            reward["title"],
            reward["prompt"],
            reward["cost"],
            reward["image"],
            reward["default_image"],
            reward["background_color"],
            reward["is_enabled"],
            reward["is_user_input_required"],
            (
                reward["max_per_stream_setting"]["is_enabled"],
                reward["max_per_stream_setting"]["max_per_stream"],
            ),
            (
                reward["max_per_user_per_stream_setting"]["is_enabled"],
                reward["max_per_user_per_stream_setting"]["max_per_user_per_stream"],
            ),
            (
                reward["global_cooldown_setting"]["is_enabled"],
                reward["global_cooldown_setting"]["global_cooldown_seconds"],
            ),
            reward["is_paused"],
            reward["is_in_stock"],
            reward["should_redemptions_skip_request_queue"],
            reward["redemptions_redeemed_current_stream"],
            datetime.strptime(reward["cooldown_expires_at"], date.RFC3339_FORMAT),
        )
        for reward in rewards
    ]


def get_custom_reward_redemption(
    token: str,
    client_id: str,
    broadcaster_id: str,
    reward_id: str,
    redemption_ids: list[str] | None = None,
    status: str | None = None,
    sort: str = "OLDEST",
    first: int = 20,
) -> list[Redemption]:
    url = "https://api.twitch.tv/helix/channel_points/custom_rewards/redemptions"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {}
    params["broadcaster_id"] = broadcaster_id
    params["reward_id"] = reward_id
    params["sort"] = sort

    if redemption_ids is not None and len(redemption_ids) > 0:
        params["id"] = redemption_ids

    if status is not None:
        params["status"] = status

    redemptions = http.send_get_with_pagination(url, headers, params, first, 50)

    return [
        Redemption(
            Channel(
                User(
                    redemption["broadcaster_id"],
                    redemption["broadcaster_login"],
                    redemption["broadcaster_name"],
                )
            ),
            redemption["id"],
            User(
                redemption["user_id"], redemption["user_login"], redemption["user_name"]
            ),
            redemption["user_input"],
            redemption["status"],
            datetime.strptime(redemption["redeemed_at"], date.RFC3339_FORMAT),
            Reward(
                Channel(
                    User(
                        redemption["broadcaster_id"],
                        redemption["broadcaster_login"],
                        redemption["broadcaster_name"],
                    )
                ),
                redemption["reward"]["id"],
                redemption["reward"]["title"],
                redemption["reward"]["prompt"],
                redemption["reward"]["cost"],
            ),
        )
        for redemption in redemptions
    ]


def update_custom_reward(
    token: str,
    client_id: str,
    broadcaster_id: str,
    reward_id: str,
    title: str | None = None,
    prompt: str | None = None,
    cost: int | None = None,
    background_color: str | None = None,
    is_enabled: bool | None = None,
    is_user_input_required: bool | None = None,
    is_max_per_stream_enabled: bool | None = None,
    max_per_stream: int | None = None,
    is_max_per_user_per_stream_enabled: bool | None = None,
    max_per_user_per_stream: int | None = None,
    is_global_cooldown_enabled: bool | None = None,
    global_cooldown_seconds: int | None = None,
    is_paused: bool | None = None,
    should_redemptions_skip_request_queue: bool | None = None,
) -> Reward:
    url = ENDPOINT_CUSTOM_REWARDS
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    data = {}
    data["broadcaster_id"] = broadcaster_id
    data["id"] = reward_id

    if title is not None:
        data["title"] = title

    if prompt is not None:
        data["prompt"] = prompt

    if cost is not None:
        data["cost"] = cost

    if background_color is not None:
        data["background_color"] = background_color

    if is_enabled is not None:
        data["is_enabled"] = is_enabled

    if is_user_input_required is not None:
        data["is_user_input_required"] = is_user_input_required

    if is_max_per_stream_enabled is not None:
        data["is_max_per_stream_enabled"] = is_max_per_stream_enabled

    if max_per_stream is not None:
        data["max_per_stream"] = max_per_stream

    if is_max_per_user_per_stream_enabled is not None:
        data["is_max_per_user_per_stream_enabled"] = is_max_per_user_per_stream_enabled

    if max_per_user_per_stream is not None:
        data["max_per_user_per_stream"] = max_per_user_per_stream

    if is_global_cooldown_enabled is not None:
        data["is_global_cooldown_enabled"] = is_global_cooldown_enabled

    if global_cooldown_seconds is not None:
        data["global_cooldown_seconds"] = global_cooldown_seconds

    if is_paused is not None:
        data["is_paused"] = is_paused

    if should_redemptions_skip_request_queue is not None:
        data["should_redemptions_skip_request_queue"] = (
            should_redemptions_skip_request_queue
        )

    reward = http.send_patch_get_result(url, headers, data)[0]

    return Reward(
        Channel(
            User(
                reward["broadcaster_id"],
                reward["broadcaster_login"],
                reward["broadcaster_name"],
            )
        ),
        reward["id"],
        reward["title"],
        reward["prompt"],
        reward["cost"],
        reward["image"],
        reward["default_image"],
        reward["background_color"],
        reward["is_enabled"],
        reward["is_user_input_required"],
        (
            reward["max_per_stream_setting"]["is_enabled"],
            reward["max_per_stream_setting"]["max_per_stream"],
        ),
        (
            reward["max_per_user_per_stream_setting"]["is_enabled"],
            reward["max_per_user_per_stream_setting"]["max_per_user_per_stream"],
        ),
        (
            reward["global_cooldown_setting"]["is_enabled"],
            reward["global_cooldown_setting"]["global_cooldown_seconds"],
        ),
        reward["is_paused"],
        reward["is_in_stock"],
        reward["should_redemptions_skip_request_queue"],
        reward["redemptions_redeemed_current_stream"],
        datetime.strptime(reward["cooldown_expires_at"], date.RFC3339_FORMAT),
    )


def update_redemption_status(
    token: str,
    client_id: str,
    redemption_id: list[str],
    broadcaster_id: str,
    reward_id: str,
    status: str | None = None,
) -> list[Redemption]:
    url = "https://api.twitch.tv/helix/channel_points/custom_rewards/redemptions"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    data = {
        "id": redemption_id,
        "broadcaster_id": broadcaster_id,
        "reward_id": reward_id,
    }

    if status is not None:
        data["status"] = status

    redemptions = http.send_patch_get_result(url, headers, data)

    return [
        Redemption(
            Channel(
                User(
                    redemption["broadcaster_id"],
                    redemption["broadcaster_login"],
                    redemption["broadcaster_name"],
                )
            ),
            redemption["id"],
            User(
                redemption["user_id"], redemption["user_login"], redemption["user_name"]
            ),
            redemption["user_input"],
            redemption["status"],
            datetime.strptime(redemption["redeemed_at"], date.RFC3339_FORMAT),
            Reward(
                Channel(
                    User(
                        redemption["broadcaster_id"],
                        redemption["broadcaster_login"],
                        redemption["broadcaster_name"],
                    )
                ),
                redemption["reward"]["id"],
                redemption["reward"]["title"],
                redemption["reward"]["prompt"],
                redemption["reward"]["cost"],
            ),
        )
        for redemption in redemptions
    ]



from datetime import datetime

from .._utils import date, http
from ..dataclasses import Conduit, ConduitShard, EventSubSubscription, Transport

ENDPOINT_CONDUITS = "https://api.twitch.tv/helix/eventsub/conduits"
ENDPOINT_SUBSCRIPTIONS = "https://api.twitch.tv/helix/eventsub/subscriptions"

CONTENT_TYPE_APPLICATION_JSON = "application/json"


def get_conduits(token: str, client_id: str) -> list[Conduit]:
    url = ENDPOINT_CONDUITS
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }

    conduits = http.send_get(url, headers, {})

    return [Conduit(conduit["id"], conduit["shard_count"]) for conduit in conduits]


def create_conduits(token: str, client_id: str, shard_count: int) -> Conduit:
    url = ENDPOINT_CONDUITS
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
        "Content-Type": CONTENT_TYPE_APPLICATION_JSON,
    }
    payload = {"shard_count": shard_count}

    conduit = http.send_post_get_result(url, headers, payload)[0]

    return Conduit(conduit["id"], conduit["shard_count"])


def update_conduits(
    token: str, client_id: str, conduit_id: str, shard_count: int
) -> Conduit:
    url = ENDPOINT_CONDUITS
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
        "Content-Type": CONTENT_TYPE_APPLICATION_JSON,
    }
    data = {"id": conduit_id, "shard_count": shard_count}

    conduit = http.send_patch_get_result(url, headers, data)[0]

    return Conduit(conduit["id"], conduit["shard_count"])


def delete_conduit(token: str, client_id: str, conduit_id: str) -> None:
    url = ENDPOINT_CONDUITS
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    data = {"id": conduit_id}

    http.send_delete(url, headers, data)


def get_conduit_shards(
    token: str, client_id: str, conduit_id: str, status: str | None = None
) -> list[ConduitShard]:
    url = "https://api.twitch.tv/helix/eventsub/conduits/shards"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {"conduit_id": conduit_id}

    if status is not None:
        params["status"] = status

    conduit_shards = http.send_get_with_infinite_pagination(url, headers, params)

    return [
        ConduitShard(
            shard["id"],
            shard["status"],
            Transport(
                shard["transport"]["method"],
                shard["transport"]["callback"],
                shard["transport"]["session_id"],
                datetime.strptime(
                    shard["transport"]["connected_at"], date.RFC3339_FORMAT
                ),
                datetime.strptime(
                    shard["transport"]["disconnected_at"], date.RFC3339_FORMAT
                ),
            ),
        )
        for shard in conduit_shards
    ]


def update_conduit_shards(
    token: str,
    client_id: str,
    conduit_id: str,
    shards: list[ConduitShard],
    session_id: str | None = None,
) -> list[ConduitShard]:
    url = "https://api.twitch.tv/helix/eventsub/conduits/shards"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
        "Content-Type": CONTENT_TYPE_APPLICATION_JSON,
    }
    data = {"conduit_id": conduit_id, "shards": shards}

    if session_id is not None:
        data["session_id"] = session_id

    conduit_shards = http.send_patch_get_result(url, headers, data)

    return [
        ConduitShard(
            shard["id"],
            shard["status"],
            Transport(
                shard["transport"]["method"],
                shard["transport"]["callback"],
                shard["transport"]["session_id"],
                datetime.strptime(
                    shard["transport"]["connected_at"], date.RFC3339_FORMAT
                ),
                datetime.strptime(
                    shard["transport"]["disconnected_at"], date.RFC3339_FORMAT
                ),
            ),
        )
        for shard in conduit_shards
    ]


def create_eventsub_subscription(
    token: str,
    client_id: str,
    subscription_type: str,
    version: str,
    condition: dict,
    transport: Transport,
) -> EventSubSubscription:
    url = ENDPOINT_SUBSCRIPTIONS
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
        "Content-Type": CONTENT_TYPE_APPLICATION_JSON,
    }
    payload = {
        "type": subscription_type,
        "version": version,
        "condition": condition,
        "transport": transport,
    }

    subscription = http.send_post_get_result(url, headers, payload)[0]

    return EventSubSubscription(
        subscription["id"],
        subscription["status"],
        subscription["type"],
        subscription["version"],
        subscription["condition"],
        subscription["created_at"],
        Transport(
            subscription["transport"]["method"],
            subscription["transport"]["callback"],
            subscription["transport"]["session_id"],
            datetime.strptime(
                subscription["transport"]["connected_at"], date.RFC3339_FORMAT
            ),
            conduit_id=subscription["transport"]["conduit_id"],
        ),
        subscription["cost"],
    )


def delete_eventsub_subscription(
    token: str, client_id: str, subscription_id: str
) -> None:
    url = ENDPOINT_SUBSCRIPTIONS
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    data = {"id": subscription_id}

    http.send_delete(url, headers, data)


def get_eventsub_subscriptions(
    token: str,
    client_id: str,
    status: str | None = None,
    subscription_type: str | None = None,
    user_id: str | None = None,
) -> list[EventSubSubscription]:
    url = ENDPOINT_SUBSCRIPTIONS
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {}

    if status is not None:
        params["status"] = status

    if subscription_type is not None:
        params["type"] = subscription_type

    if user_id is not None:
        params["user_id"] = user_id

    subscriptions = http.send_get(url, headers, params)

    return [
        EventSubSubscription(
            subscription["id"],
            subscription["status"],
            subscription["type"],
            subscription["version"],
            subscription["condition"],
            datetime.strptime(subscription["created_at"], date.RFC3339_FORMAT),
            Transport(
                subscription["transport"]["method"],
                subscription["transport"]["callback"],
                subscription["transport"]["session_id"],
                datetime.strptime(
                    subscription["transport"]["connected_at"], date.RFC3339_FORMAT
                ),
                datetime.strptime(
                    subscription["transport"]["disconnected_at"], date.RFC3339_FORMAT
                ),
            ),
            subscription["cost"],
        )
        for subscription in subscriptions
    ]



from datetime import datetime

from .._utils import date, http
from ..dataclasses import AdSchedule, Commercial


def start_commercial(
    token: str, client_id: str, broadcaster_id: int, length: int
) -> Commercial:
    url = "https://api.twitch.tv/helix/channels/commercial"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
        "Content-Type": "application/json",
    }
    payload = {"broadcaster_id": broadcaster_id, "length": length}

    commercial = http.send_post_get_result(url, headers, payload)[0]

    return Commercial(
        commercial["length"], commercial["message"], commercial["retry_after"]
    )


def get_ad_schedule(token: str, client_id: str, broadcaster_id: str) -> AdSchedule:
    url = "https://api.twitch.tv/helix/channels/ads"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {"broadcaster_id": broadcaster_id}

    ad_schedule = http.send_get(url, headers, params)[0]

    return AdSchedule(
        ad_schedule["snooze_count"],
        datetime.strptime(ad_schedule["snooze_refresh_at"], date.RFC3339_FORMAT),
        datetime.strptime(ad_schedule["next_ad_at"], date.RFC3339_FORMAT),
        ad_schedule["duration"],
        datetime.strptime(ad_schedule["last_ad_at"], date.RFC3339_FORMAT),
        ad_schedule["preroll_free_time"],
    )


def snooze_next_ad(token: str, client_id: str, broadcaster_id: str) -> AdSchedule:
    url = "https://api.twitch.tv/helix/channels/ads/schedule/snooze"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    payload = {"broadcaster_id": broadcaster_id}

    ad_schedule = http.send_post_get_result(url, headers, payload)[0]

    return AdSchedule(
        ad_schedule["snooze_count"],
        datetime.strptime(ad_schedule["snooze_refresh_at"], date.RFC3339_FORMAT),
        datetime.strptime(ad_schedule["next_ad_at"], date.RFC3339_FORMAT),
    )



from datetime import datetime

from .._utils import date, http
from ..dataclasses import Channel, ContentClassificationLabel, Game, User

ENDPOINT_VIPS = "https://api.twitch.tv/helix/channels/vips"


def get_channel_information(
    token: str, client_id: str, broadcaster_id: list[str]
) -> list[Channel]:
    url = "https://api.twitch.tv/helix/channels"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {"broadcaster_id": broadcaster_id}

    channels = http.send_get(url, headers, params)

    return [
        Channel(
            User(
                channel["broadcaster_id"],
                channel["broadcaster_login"],
                channel["broadcaster_name"],
            ),
            channel["broadcaster_language"],
            Game(channel["game_id"], channel["game_name"]),
            channel["title"],
            channel["tags"],
            channel["delay"],
            channel["content_classification_labels"],
            channel["is_branded_content"],
        )
        for channel in channels
    ]


def modify_channel_information(
    token: str,
    client_id: str,
    broadcaster_id: str,
    game_id: str | None = None,
    broadcaster_language: str | None = None,
    title: str | None = None,
    delay: int | None = None,
    tags: list[str] | None = None,
    content_classification_labels: list[ContentClassificationLabel] | None = None,
    is_branded_content: bool | None = None,
) -> None:
    url = "https://api.twitch.tv/helix/channels"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    data = {}
    data["broadcaster_id"] = broadcaster_id

    if game_id is not None:
        data["game_id"] = game_id

    if broadcaster_language is not None:
        data["broadcaster_language"] = broadcaster_language

    if title is not None:
        data["title"] = title

    if delay is not None:
        data["delay"] = delay

    if tags is not None and len(tags) > 0:
        data["tags"] = tags

    if (
        content_classification_labels is not None
        and len(content_classification_labels) > 0
    ):
        data["content_classification_labels"] = content_classification_labels

    if is_branded_content is not None:
        data["is_branded_content"] = is_branded_content

    http.send_patch(url, headers, data)


def get_channel_editors(token: str, client_id: str, broadcaster_id: str) -> list[User]:
    url = "https://api.twitch.tv/helix/channels/editors"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {"broadcaster_id": broadcaster_id}

    editors = http.send_get(url, headers, params)

    return [
        User(
            editor["user_id"],
            editor["user_name"].lower(),
            editor["user_name"],
            created_at=datetime.strptime(editor["created_at"], date.RFC3339_FORMAT),
        )
        for editor in editors
    ]


def get_followed_channels(
    token: str,
    client_id: str,
    user_id: str,
    broadcaster_id: str | None = None,
    first: int = 20,
) -> list[tuple[Channel, datetime]]:
    url = "https://api.twitch.tv/helix/channels/followed"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {"user_id": user_id}

    if broadcaster_id is not None:
        params["broadcaster_id"] = broadcaster_id

    followed_channels = http.send_get_with_pagination(url, headers, params, first, 100)

    return [
        (
            Channel(
                User(
                    followed_channel["broadcaster_id"],
                    followed_channel["broadcaster_login"],
                    followed_channel["broadcaster_name"],
                )
            ),
            datetime.strptime(followed_channel["followed_at"], date.RFC3339_FORMAT),
        )
        for followed_channel in followed_channels
    ]


def get_channel_followers(
    token: str,
    client_id: str,
    broadcaster_id: str,
    user_id: str | None = None,
    first: int = 20,
) -> list[tuple[Channel, datetime]]:
    url = "https://api.twitch.tv/helix/channels/followers"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {"broadcaster_id": broadcaster_id}

    if user_id is not None:
        params["user_id"] = user_id

    followers = http.send_get_with_pagination(url, headers, params, first, 100)

    return [
        (
            Channel(
                User(follower["user_id"], follower["user_login"], follower["user_name"])
            ),
            datetime.strptime(follower["followed_at"], date.RFC3339_FORMAT),
        )
        for follower in followers
    ]


def get_vips(
    token: str,
    client_id: str,
    broadcaster_id: str,
    user_id: list[str] | None = None,
    first: int = 20,
) -> list[User]:
    url = ENDPOINT_VIPS
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {}
    params["broadcaster_id"] = broadcaster_id

    if user_id is not None and len(user_id) > 0:
        params["user_id"] = user_id

    vips = http.send_get_with_pagination(url, headers, params, first, 20)

    return [User(vip["user_id"], vip["user_login"], vip["user_name"]) for vip in vips]


def add_channel_vip(
    token: str, client_id: str, user_id: str, broadcaster_id: str
) -> None:
    url = ENDPOINT_VIPS
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    payload = {"user_id": user_id, "broadcaster_id": broadcaster_id}

    http.send_post(url, headers, payload)


def remove_channel_vip(
    token: str, client_id: str, user_id: str, broadcaster_id: str
) -> None:
    url = ENDPOINT_VIPS
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    data = {"user_id": user_id, "broadcaster_id": broadcaster_id}

    http.send_delete(url, headers, data)



from dataclasses import dataclass
from datetime import datetime

from ..dataclasses import Transport


@dataclass
class EventSubSubscription:
    """
    Represents an EventSub subscription

    Attributes:
        id (str): An ID that identifies the subscription
        status (str): The subscription’s status
            Possible values are: enabled, webhook_callback_verification_pending, webhook_callback_verification_failed, notification_failures_exceeded, authorization_revoked, user_removed
        type (str): The subscription’s type
        version (str): The version of the subscription type
        condition (dict): The subscription’s parameter values
        created_at (str): The RFC 3339 timestamp indicating when the subscription was created
        transport (Transport): The transport details used to send you notifications
        cost (int): The amount that the subscription counts against your limit
    """

    subscription_id: str
    status: str
    subscription_type: str
    version: str
    condition: dict
    created_at: datetime
    transport: Transport
    cost: int



from dataclasses import dataclass
from datetime import datetime

from ..dataclasses import HypeTrainContribution


@dataclass
class HypeTrainEventData:
    """
    Represents the data of a Hype Train's event

    Attributes:
        broadcaster_id (str): The ID of the broadcaster that’s running the Hype Train
        cooldown_end_time (datetime): The UTC date and time (in RFC3339 format) that another Hype Train can start
        expires_at (datetime): The UTC date and time (in RFC3339 format) that the Hype Train ends
        goal (int): The value needed to reach the next level
        hype_train_id (str): An ID that identifies this Hype Train
        last_contribution (HypeTrainContribution): The most recent contribution towards the Hype Train’s goal
        level (int): The highest level that the Hype Train reached (the levels are 1 through 5)
        started_at (datetime): The UTC date and time (in RFC3339 format) that this Hype Train started
        top_contributions (list[HypeTrainContribution]): The top contributors for each contribution type
        total (int): The current total amount raised
    """

    broadcaster_id: str
    cooldown_end_time: datetime
    expires_at: datetime
    goal: int
    hype_train_id: str
    last_contribution: HypeTrainContribution
    level: int
    started_at: datetime
    top_contributions: list[HypeTrainContribution]
    total: int



from dataclasses import dataclass


@dataclass
class ChatSettings:
    """
    Represents a chat's settings

    Attributes:
        broadcaster_id (str): The ID of the broadcaster specified in the request
        emote_mode (bool): A Boolean value that determines whether chat messages must contain only emotes
        follower_mode (bool): A Boolean value that determines whether the broadcaster restricts the chat room to followers only
        follower_mode_duration (int): The length of time, in minutes, that users must follow the broadcaster before being able to participate in the chat room
        moderator_id (str): The moderator’s ID
        non_moderator_chat_delay (bool): A Boolean value that determines whether the broadcaster adds a short delay before chat messages appear in the chat room
        non_moderator_chat_delay_duration (int): The amount of time, in seconds, that messages are delayed before appearing in chat
        slow_mode (bool): A Boolean value that determines whether the broadcaster limits how often users in the chat room are allowed to send messages
        slow_mode_wait_time (int): The amount of time, in seconds, that users must wait between sending messages
        subscriber_mode (bool): A Boolean value that determines whether only users that subscribe to the broadcaster’s channel may talk in the chat room
        unique_chat_mode (bool): A Boolean value that determines whether the broadcaster requires users to post only unique messages in the chat room
    """

    broadcaster_id: str
    emote_mode: bool
    follower_mode: bool
    follower_mode_duration: int
    moderator_id: str
    non_moderator_chat_delay: bool
    non_moderator_chat_delay_duration: int
    slow_mode: bool
    slow_mode_wait_time: int
    subscriber_mode: bool
    unique_chat_mode: bool



from dataclasses import dataclass
from datetime import datetime

from ..dataclasses import User


@dataclass
class Guest:
    """
    Represents a guest in a Guest Star session

    Attributes:
        slot_id (str): ID representing this guest’s slot assignment
        is_live (bool): Flag determining whether or not the guest is visible in the browser source in the host’s streaming software
        user (User): The user assigned to this slot
        volume (int): Value from 0 to 100 representing the host’s volume setting for this guest
        assigned_at (datetime): Timestamp when this guest was assigned a slot in the session
        audio_settings (dict): Information about the guest’s audio settings
        video_settings (dict): Information about the guest’s video settings
    """

    slot_id: str
    is_live: bool
    user: User
    volume: int
    assigned_at: datetime
    audio_settings: dict
    video_settings: dict



from dataclasses import dataclass


@dataclass
class Commercial:
    """
    Represents a commercial

    Attributes:
        length (int): The length of the commercial
            Maximum: 180
        message (str): A message that indicates whether Twitch was able to serve an ad
        retry_after (int): The number of seconds you must wait before running another commercial
    """

    length: int
    message: str
    retry_after: int



from dataclasses import dataclass
from datetime import datetime

from ..dataclasses import Channel, PollChoice


@dataclass
class Poll:
    """
    Represents a poll

    Args:
        poll_id (str): ID of the poll
        broadcaster_id (str): ID of the broadcaster
        broadcaster_name (str): Name of the broadcaster
        broadcaster_login (str): Login of the broadcaster
        title (str): Question displayed for the poll
        choices (list): The poll choices
        channel_points_voting_enabled (bool): Indicates if Channel Points can be used for voting
        channel_points_per_vote (int): Number of Channel Points required to vote once with Channel Points
        status (str): Poll status
            Valid values are: ACTIVE, COMPLETED, TERMINATED, ARCHIVED, MODERATED, INVALID
        duration (int): Total duration for the poll (in seconds)
        started_at (str): UTC timestamp for the poll’s start time
        ended_at (str): UTC timestamp for the poll’s end time
    """

    poll_id: str
    channel: Channel
    title: str
    choices: list[PollChoice]
    channel_points_voting_enabled: bool
    channel_points_per_vote: int
    status: str
    duration: int
    started_at: datetime
    ended_at: datetime | None = None



from .._utils import http
from ..dataclasses import Game


def get_top_games(token: str, client_id: str, first: int = 20) -> list[Game]:
    url = "https://api.twitch.tv/helix/games/top"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }

    games = http.send_get_with_pagination(url, headers, {}, first, 100)

    return [
        Game(
            game["id"],
            game["name"],
            game["box_art_url"],
            game["igdb_id"],
        )
        for game in games
    ]


def get_games(
    token: str,
    client_id: str,
    game_id: list[str] | None = None,
    name: list[str] | None = None,
    igdb_id: list[str] | None = None,
) -> list[Game]:
    url = "https://api.twitch.tv/helix/games"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {}

    if game_id is not None and len(game_id) > 0:
        params["id"] = game_id

    if name is not None and len(name) > 0:
        params["name"] = name

    if igdb_id is not None and len(igdb_id) > 0:
        params["igdb_id"] = igdb_id

    games = http.send_get(url, headers, params)

    return [
        Game(game["id"], game["name"], game["box_art_url"], game["igdb_id"])
        for game in games
    ]



from dataclasses import dataclass
from datetime import datetime

from ..dataclasses import Channel, Reward, User


@dataclass
class Redemption:
    """
    Represents a reward redemption

    Args:
        channel (Channel): The channel that the reward belongs to
        redemption_id (str): The ID of the redemption
        user (User): The user that redeemed the reward
        user_input (str): The user input provided
        status (str): One of UNFULFILLED, FULFILLED or CANCELED
        redeemed_at (datetime): Timestamp of when the reward was redeemed
        reward (Reward): The custom reward that was redeemed at the time it was redeemed
    """

    channel: Channel
    redemption_id: str
    user: User
    user_input: str
    status: str
    redeemed_at: datetime
    reward: Reward



from dataclasses import dataclass

from ..dataclasses import Channel, CharityCampaignAmount


@dataclass
class CharityCampaign:
    """
    Represents a charity campaign

    Attributes:
        campaign_id (str): An ID that identifies the charity campaign
        channel (Channel): The channel that's running the campaign
        charity_name (str): The charity’s name
        charity_description (str): A description of the charity
        charity_logo (str): A URL to an image of the charity’s logo
        charity_website (str): A URL to the charity’s website
        current_amount (CharityCampaignAmount): The current amount of donations that the campaign has received
        target_amount (CharityCampaignAmount): The campaign’s fundraising goal
    """

    campaign_id: str
    channel: Channel
    charity_name: str
    charity_description: str
    charity_logo: str
    charity_website: str
    current_amount: CharityCampaignAmount
    target_amount: CharityCampaignAmount



from dataclasses import dataclass
from datetime import datetime

from ..dataclasses import User


@dataclass
class Team:
    """
    Represents a team

    Attributes:
        users (list[User]): Users in the team
        background_image_url (str): URL of the team background image
        banner (str): URL for the team banner
        created_at (datetime): Date and time the team was created
        updated_at (datetime): Date and time the team was last updated
        info (str): Team description
        thumbnail_url (str): Image URL for the team logo
        team_name (str): Team name
        team_display_name (str): Team display name
        team_id (str): Team ID
    """

    users: list[User]
    background_image_url: str
    banner: str
    created_at: datetime
    updated_at: datetime
    info: str
    thumbnail_url: str
    team_name: str
    team_display_name: str
    team_id: str



from datetime import datetime

from .._utils import date, http
from ..dataclasses import DropEntitlement


def get_drops_entitlements(
    token: str,
    client_id: str,
    entitlement_id: list[str] | None = None,
    user_id: str | None = None,
    game_id: str | None = None,
    fulfillment_status: str | None = None,
    first: int = 20,
) -> list[DropEntitlement]:
    url = "https://api.twitch.tv/helix/entitlements/drops"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {}

    if entitlement_id is not None:
        params["id"] = entitlement_id

    if user_id is not None:
        params["user_id"] = user_id

    if game_id is not None:
        params["game_id"] = game_id

    if fulfillment_status is not None:
        params["fulfillment_status"] = fulfillment_status

    drops = http.send_get_with_pagination(url, headers, params, first, 1000)

    return [
        DropEntitlement(
            drop["id"],
            drop["benefit_id"],
            datetime.strptime(drop["timestamp"], date.RFC3339_FORMAT),
            drop["user_id"],
            drop["game_id"],
            drop["fulfillment_status"],
            datetime.strptime(drop["last_updated"], date.RFC3339_FORMAT),
        )
        for drop in drops
    ]


def update_drops_entitlements(
    token: str,
    client_id: str,
    entitlement_ids: list[str] | None = None,
    fulfillment_status: str | None = None,
) -> list[tuple[str, list[str]]]:
    url = "https://api.twitch.tv/helix/entitlements/drops"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
        "Content-Type": "application/json",
    }
    data = {}

    if entitlement_ids is not None and len(entitlement_ids) > 0:
        data["entitlement_ids"] = entitlement_ids

    if fulfillment_status is not None:
        data["fulfillment_status"] = fulfillment_status

    drops_updates = http.send_patch_get_result(url, headers, data)

    return [
        (drop_update["status"], drop_update["ids"]) for drop_update in drops_updates
    ]



from dataclasses import dataclass


@dataclass
class ExtensionConfigurationSegment:
    """
    Represents a configuration segment of an extension

    Attributes:
        segment (str): The type of segment
            Possible values: broadcaster, developer, global
        broadcaster_id (str): The ID of the broadcaster that installed the extension
        content (str): The contents of the segment
        version (str): The version number that identifies this definition of the segment’s data
    """

    segment: str
    broadcaster_id: str
    content: str
    version: str



from datetime import datetime

from .._utils import date, http
from ..dataclasses import ExtensionAnalyticsReport, GameAnalyticsReport


def get_extension_analytics(
    token: str,
    client_id: str,
    extension_id: str | None = None,
    report_type: str | None = None,
    started_at: datetime | None = None,
    ended_at: datetime | None = None,
    first: int = 20,
) -> list[ExtensionAnalyticsReport]:
    url = "https://api.twitch.tv/helix/analytics/extensions"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {}

    if extension_id is not None:
        params["extension_id"] = extension_id

    if report_type is not None:
        params["type"] = report_type

    if started_at is not None:
        params["started_at"] = started_at

    if ended_at is not None:
        params["ended_at"] = ended_at

    reports = http.send_get_with_pagination(url, headers, params, first, 100)

    return [
        ExtensionAnalyticsReport(
            report["extension_id"],
            report["URL"],
            report["type"],
            datetime.strptime(report["date_range"]["started_at"], date.RFC3339_FORMAT),
            datetime.strptime(report["date_range"]["ended_at"], date.RFC3339_FORMAT),
        )
        for report in reports
    ]


def get_game_analytics(
    token: str,
    client_id: str,
    game_id: str | None = None,
    report_type: str | None = None,
    started_at: datetime | None = None,
    ended_at: datetime | None = None,
    first: int = 20,
) -> list[GameAnalyticsReport]:
    url = "https://api.twitch.tv/helix/analytics/games"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {}

    if game_id is not None:
        params["game_id"] = game_id

    if report_type is not None:
        params["type"] = report_type

    if started_at is not None:
        params["started_at"] = started_at

    if ended_at is not None:
        params["ended_at"] = ended_at

    reports = http.send_get_with_pagination(url, headers, params, first, 100)

    return [
        GameAnalyticsReport(
            report["game_id"],
            report["URL"],
            report["type"],
            datetime.strptime(report["date_range"]["started_at"], date.RFC3339_FORMAT),
            datetime.strptime(report["date_range"]["ended_at"], date.RFC3339_FORMAT),
        )
        for report in reports
    ]



from distutils.core import setup

setup(
    name="twitchpy",
    packages=["twitchpy", "twitchpy._api", "twitchpy._utils", "twitchpy.dataclasses"],
    version="1.2.2",
    license="GPL-3.0",
    description="TwitchPy is a Python package for using the Twitch's API and create bots for interacting with their IRC chats.",
    author="DaCasBe",
    author_email="dacasbe97@gmail.com",
    url="https://github.com/DaCasBe",
    download_url="https://github.com/DaCasBe/TwitchPy/archive/refs/tags/v1.2.0.tar.gz",
    install_requires=["requests"],
)



from datetime import datetime

from .._utils import date, http


def start_raid(
    token: str, client_id: str, from_broadcaster_id: str, to_broadcaster_id: str
) -> tuple[datetime, bool]:
    url = "https://api.twitch.tv/helix/raids"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    payload = {
        "from_broadcaster_id": from_broadcaster_id,
        "to_broadcaster_id": to_broadcaster_id,
    }

    raid = http.send_post_get_result(url, headers, payload)[0]

    return (
        datetime.strptime(raid["created_at"], date.RFC3339_FORMAT),
        raid["is_mature"],
    )


def cancel_raid(token: str, client_id: str, broadcaster_id: str) -> None:
    url = "https://api.twitch.tv/helix/raids"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    data = {"broadcaster_id": broadcaster_id}

    http.send_delete(url, headers, data)



from datetime import datetime

from .._utils import http
from ..dataclasses import Channel, Clip, User


def create_clip(
    token: str, client_id: str, broadcaster_id: str, has_delay: bool = False
) -> tuple[str, str]:
    url = "https://api.twitch.tv/helix/clips"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    payload = {"broadcaster_id": broadcaster_id, "has_delay": has_delay}

    clip_creation = http.send_post_get_result(url, headers, payload)[0]

    return (clip_creation["id"], clip_creation["edit_url"])


def get_clips(
    token: str,
    client_id: str,
    broadcaster_id: str | None = None,
    game_id: str | None = None,
    clip_ids: list[str] | None = None,
    started_at: datetime | None = None,
    ended_at: datetime | None = None,
    first: int = 20,
    is_featured: bool | None = None,
) -> list[Clip]:
    url = "https://api.twitch.tv/helix/clips"
    headers = {
        "Authorization": f"Bearer {token}",
        "Client-Id": client_id,
    }
    params = {}

    if broadcaster_id is not None:
        params["broadcaster_id"] = broadcaster_id

    if game_id is not None:
        params["game_id"] = game_id

    if clip_ids is not None and len(clip_ids) > 0:
        params["id"] = clip_ids

    if started_at is not None:
        params["started_at"] = started_at

    if ended_at is not None:
        params["ended_at"] = ended_at

    if is_featured is not None:
        params["is_featured"] = is_featured

    clips = http.send_get_with_pagination(url, headers, params, first, 100)

    return [
        Clip(
            clip["id"],
            clip["url"],
            clip["embed_url"],
            Channel(
                User(
                    clip["broadcaster_id"],
                    clip["broadcaster_name"].lower(),
                    clip["broadcaster_name"],
                )
            ),
            User(
                clip["creator_id"], clip["creator_name"].lower(), clip["creator_name"]
            ),
            clip["video_id"],
            clip["game_id"],
            clip["language"],
            clip["title"],
            clip["view_count"],
            clip["created_at"],
            clip["thumbnail_url"],
            clip["duration"],
            clip["vod_offset"],
            clip["is_featured"],
        )
        for clip in clips
    ]



from dataclasses import dataclass

from ..dataclasses import User


@dataclass
class BitsLeaderboardLeader:
    """
    Represents a user in a Bits Leaderboard

    Attributes:
        user (User): The user on the leaderboard
        rank (int): The user’s position on the leaderboard
        score (int): The number of Bits the user has cheered
    """

    user: User
    rank: int
    score: int
